#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "debug.h"
#include "trace.h"
#ifdef WIN32
#define inline	__inline
#endif

/*
 * check if the trace mode is valid
 */
inline int is_valid_mode(int mode) {
	if(mode == DISKMON_TRACE) return 1;

	return 0;
}

/*
 * check if the operation is valid
 */
inline int is_valid_op(int op) {
	if(op == INV) return 0;
	else if(op == READ || op == WRITE || op == TRIM)
		return 1;
	return 0;
}

/*
 * check the traceio structure is valid
 */
inline int is_valid_traceio(struct traceio *traceio){
	if(is_valid_mode(traceio->mode)) {
		if(traceio->fp != NULL) return 1;
	}

	return 0;
}

/*
 * open trace file
 */
traceio_handler open_traceio(char *filename, int mode){
#ifdef WIN32
	int err;
#endif
	struct traceio *ret;
	FILE *fp = NULL;
	ret = (struct traceio *)malloc(sizeof(struct traceio));
	if(ret == NULL){
		error("trace file open filaed: out of memory");
		goto err1;
	}

	{
		if(!is_valid_mode(mode)){
			error("trace file open failed: invalid mode");
			goto err;
		}

#ifdef WIN32
		err = fopen_s(&fp, filename, "r");
		if(err != 0) {
#else
		fp = fopen(filename, "r");
		if(fp == NULL) {
#endif
			error("trace file open failed: (%s)", filename);
			goto err;
		}


		ret->fp = fp;
	}
	ret->mode = mode;

	return ret;

err1:
	fclose(fp);
err:
	return NULL;

}

/*
 * close trace file
 */
void close_traceio(traceio_handler handler){
	struct traceio *traceio;

	traceio = (struct traceio *)handler;

	if(!is_valid_traceio(traceio)){
		error("trace close failed: invalid handler");
		return;
	}

	fclose(traceio->fp);

	free(traceio);
	return;
}

/*
 * convert operation string to number
 */
_u32 str2op(char *op){
	if(op[0] == 'R') return READ;
	else if(op[0] == 'W') return WRITE;
//	else if(op[0] == 'T') return TRIM;

	return INV;
}

struct op_unit __next_op_diskmon(struct traceio *traceio){
	char op[8];
	struct op_unit eofret = {END, 0, 0}, ret;
	double temp1, temp2, temp3, temp4;

	if(fscanf(traceio->fp, 
		"%lf\t%lf\t%lf\t%lf\t%s\t%u\t%u\n",
		&temp1, &temp2, &temp3, &temp4, op, &ret.sector, &ret.length) == EOF){
		return eofret;
	}
	
	ret.op = str2op(op);

	return ret;
}

/*
 * extract next operation information from trace file
 */
struct op_unit next_op(traceio_handler handler){
	struct traceio *traceio;
	struct op_unit invret = {INV, 0, 0};

	traceio = (struct traceio *)handler;
	if(!is_valid_traceio(traceio)){
		error("trace next failed: invalid handler");
		return invret;
	}

	switch(traceio->mode) {
	case DISKMON_TRACE:	return __next_op_diskmon(traceio); break;
	}
	
	return invret;
}

int is_valid_op_unit(struct op_unit op_unit){
	return is_valid_op(op_unit.op);
}

/*
 * check if trace file goes end of file
 */
int is_last(struct op_unit op_unit){
	if(op_unit.op == END) return 1;

	return 0;
}
