#include <stdlib.h>
#include "types.h"
#include "debug.h"
#include "nand.h"

#define MAP_INVALID		-1

/* 
 * simple page map table with all mapping in memory
 */
static struct pagemap {
	unsigned int nr_page;
	_u32 *table;
} pagemap;

/*
 * logical page number -> physical page number
 *		lpa: logical page number
 *		return physical page number on map
 */
_u32 map_find(_u32 lpa)
{
	if(lpa<0 || lpa>pagemap.nr_page) return -1;
	return pagemap.table[lpa];
}

/*
 * set physical page number for logical page number
 *		lpa: logical page number
 *		ppa: physical page number to save on map
 */
void map_update(_u32 lpa, _u32 ppa)
{
	if(lpa<0 || lpa>pagemap.nr_page) return;

	pagemap.table[lpa] = ppa;
}

void map_gc(_u32 victim, _u8 *bitmap)
{
}

/*
 * deallocate data structure
 */
void map_end()
{
	free(pagemap.table);
	pagemap.table = NULL;
}	

/*
 * allocate data structure
 */
void map_init()
{
	pagemap.nr_page = config_nand.page_per_blk 
			* (config_nand.numblks - config_nand.extrablks);
	pagemap.table = malloc(sizeof(_u32) * pagemap.nr_page);
}
