#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "types.h"
#include "debug.h"
#include "traceio.h"
#include "stat.h"
#include "nand.h"

#include "ftl.h"

/*
 * main loop for every operation on trace file.
 * get operation by next_op() and call ftl_read() or ftl_write().
 * after all, print stat of NAND.
 */
void run_trace(traceio_handler h)
{
	_t_sect start;
	_t_size size;
	int length;
	struct op_unit op_unit;
	int cnt=0;
	_t_size (*op_func)(_t_sect lsn, _t_size size);

/* count reset before run trace */
	nand_stat_reset();
	
	while(1) {
/* get next operation on trace file */
		op_unit = next_op(h);
		if(is_last(op_unit)) break;
		if(!is_valid_op_unit(op_unit)) {
			error("Invalid op_unit");
			break;
		}

		start = (op_unit.sector);
		length = (op_unit.length);

		switch(op_unit.op) {
		case READ:	
			op_func = ftl_read; 
			io_stat.cnt_read++; io_stat.size_read+=length;
			break;
		case WRITE: 
			op_func = ftl_write; 
			io_stat.cnt_write++; io_stat.size_write+=length;
			break;
		}

		while(length>0) {
/* execute FTL opeartion */
			size = op_func(start, length);
			start += size;
			length -= size;
			cnt++;
			if(cnt%10000==0) {
				fprintf(stderr, ".");
				fflush(stderr);
			}
		}
	}

/* print count of NAND operation */
	nand_stat_print();

	printf("\n");

}

void sim(char *tracefile, int repeat)
{
	int i;
	traceio_handler h;

/* initializing */
	nand_open();
	
	ftl_open();

/* run traces */

	i=0; 
	if(repeat==0) repeat=1;

	while(repeat) {
		fprintf(stderr,"\n%d running\n", i);
		printf("[%d] Simulating trace: %s\n", i, tracefile);

		i++; repeat--;

		h = open_traceio(tracefile, DISKMON_TRACE);
		if(h == NULL) {
			fprintf(stderr, "Error to open file:%s\n", tracefile);
			break;
		}
		run_trace(h);
		close_traceio(h);
	}

/* finalizaling */
	ftl_close();
	nand_close();

}

int main(int argc, char *argv[]) {
	int repeat;
	char tracefile[256];
	
	if(argc<3) {
		printf("Usage: %s <tracefile> <repeat>\n", argv[0]);
		exit(1);
	}
#ifdef WIN32
	strcpy_s(tracefile, 255, argv[1]);
#else
	strcpy(tracefile, argv[1]);
#endif
	repeat = atoi(argv[2]);

	sim(tracefile, repeat);

	return 0;

}

	
